@@ -24,6 +24,7 @@
 
 #if !GTK_CHECK_VERSION(2,13,4)
 #define gtk_widget_get_window(x) (x)->window
+#define gtk_color_selection_dialog_get_color_selection(x) (x)->colorsel
 #endif
 
 #if !GTK_CHECK_VERSION(2,15,0)
@@ -33,6 +34,8 @@
 #if GTK_CHECK_VERSION(2,17,5)
 #undef GTK_WIDGET_NO_WINDOW
 #define GTK_WIDGET_NO_WINDOW !gtk_widget_get_has_window
+#undef GTK_WIDGET_SET_FLAGS
+#define GTK_WIDGET_SET_FLAGS(x,y) gtk_widget_set_can_default(x,1)
 #endif
 
 #if GTK_CHECK_VERSION(2,17,7)
@@ -50,6 +53,34 @@
 #define GTK_WIDGET_REALIZED gtk_widget_get_realized
 #endif
 
+#if GTK_CHECK_VERSION(2,21,8)
+#undef GDK_DISPLAY
+#define GDK_DISPLAY() GDK_DISPLAY_XDISPLAY(gdk_display_get_default())
+#endif
+
+#if GTK_CHECK_VERSION(2,91,0)
+#define GTK_OBJECT
+#endif
+
+#if !GTK_CHECK_VERSION(2,91,1)
+#define gtk_window_set_has_resize_grip(x,y);
+#define GTK_COMBO_BOX_TEXT GTK_COMBO_BOX
+#endif
+
+#if GTK_CHECK_VERSION(2,91,2)
+#define gtk_combo_box_new_text gtk_combo_box_text_new
+#define gtk_combo_box_append_text gtk_combo_box_text_append_text
+#define gtk_widget_hide_all gtk_widget_hide
+#endif
+
+#if GTK_CHECK_VERSION(2,91,6)
+#define GDK_WINDOW_XWINDOW GDK_WINDOW_XID
+#endif
+
+#if GTK_CHECK_VERSION(2,91,7)
+#define gdk_window_lookup_for_display gdk_x11_window_lookup_for_display
+#endif
+
 typedef enum {
   GCIN_STATE_DISABLED = 0,
   GCIN_STATE_ENG_FULL = 1,
@@ -68,10 +99,10 @@ typedef enum {
 
 #if CLIENT_LIB
 #define p_err __gcin_p_err
-#define dbg __gcin_dbg
 #define zmalloc __gcin_zmalloc
 #endif
 
+#include "util.h"
 
 #define tmalloc(type,n)  (type*)malloc(sizeof(type) * (n))
 void *zmalloc(int n);
@@ -83,8 +114,6 @@ void *memdup(void *p, int n);
 extern Display *dpy;
 #endif
 
-void p_err(char *fmt,...);
-void dbg(char *fmt,...);
 
 extern GtkWidget *gwin0;
 extern GdkWindow *gdkwin0;
@@ -175,8 +204,13 @@ void sendkey_b5(char *bchar);
 void send_ascii(char key);
 void bell();
 void set_label_font_size(GtkWidget *label, int size);
+#if UNIX
 void send_gcin_message(Display *dpy, char *s);
+#else
+void send_gcin_message(char *s);
+#endif
 void check_CS();
+gint64 current_time();
 void get_win_size(GtkWidget *win, int *width, int *height);
 void change_win_fg_bg(GtkWidget *win, GtkWidget *label);
 void set_no_focus(GtkWidget *win);
