@@ -1,4 +1,3 @@
-#include <math.h>
 #include "gcin.h"
 #include "gtab.h"
 #include "gcin-conf.h"
@@ -24,6 +23,7 @@ int ch_pos_find(char *ch, int pos);
 void inc_gtab_usecount(char *str), ClrSelArea();
 void lookup_gtabn(char *ch, char *out);
 char *htmlspecialchars(char *s, char out[]);
+void hide_gtab_pre_sel();
 
 extern gboolean test_mode;
 
@@ -50,6 +50,15 @@ void extract_gtab_key(int start, int len, void *out)
   }
 }
 
+void extract_gbuf_str(int start, int len, char *out)
+{
+  int i;
+  out[0]=0;
+  for(i=0;i<len;i++)
+    strcat(out, gbuf[i+start].ch);
+}
+
+
 gboolean gtab_cursor_end()
 {
   return ggg.gbuf_cursor==ggg.gbufN;
@@ -67,16 +76,65 @@ void dump_gbuf()
   }
 }
 
-static char *gen_buf_str()
+static unich_t latin_chars[]=
+_L("ÀÁÂÃÄÅÆÆÇÈÉÊËÌÍÎÏÐÐÑÒÓÔÕÖØÙÚÛÜÝÞÞßàáâãäåææçèéêëìíîïððñòóôõöøùúûüýþþÿ")
+_L("ĀāĂăĄąĆćĈĉĊċČčĎďĐđĒēĔĕĖėĘęĚěĜĝĞğĠġĢģĤĥĦħĨĩĪīĬĭĮįİıĲĲĳĳĴĵĶķĸĹĺĻļĽľĿŀŁł")
+_L("ŃńŅņŇňŉŊŋŌōŎŏŐőŒŒœœŔŕŖŗŘřŚśŜŝŞşŠšŢţŤťŦŧŨũŪūŬŭŮůŰűŲųŴŵŶŷŸŹźŻżŽž");
+
+int en_word_len(char *bf)
+{
+  int i;
+  char *s;
+
+  for(s=bf;*s;) {
+    int sz = utf8_sz(s);
+    if (sz==1) {
+      if (!(*s >= 'A' && *s<='Z' || *s >= 'a' && *s<='z' || strchr("-_'", *s)))
+        break;
+    } else
+    if (sz==2) {
+      char *p;
+#if WIN32
+      p = _(latin_chars);
+      for (; *p; p+=2)
+#else
+      for (p=latin_chars; *p; p+=2)
+#endif
+        if (!memcmp(p, s, 2))
+          break;
+      if (!(*p))
+        break;
+    } else
+    if (sz>=3)
+      break;
+    s+=sz;
+  }
+
+  if (*s)
+    return 0;
+  return strlen(bf);
+}
+
+static char *gen_buf_str(int start, gboolean add_spc)
 {
   int i;
   char *out = tmalloc(char, 1);
   int outN=0;
 
-  for(i=0;i<ggg.gbufN;i++) {
+  gboolean last_en_word = FALSE;
+  for(i=start;i<ggg.gbufN;i++) {
     char *t = gbuf[i].ch;
     int len = strlen(t);
 
+    if (add_spc && en_word_len(t) && !(gbuf[i].flag & FLAG_CHPHO_GTAB_BUF_EN_NO_SPC)) {
+      if (last_en_word) {
+        out = trealloc(out, char, outN+1);
+        out[outN++]=' ';
+      }
+      last_en_word = TRUE;
+    } else
+      last_en_word = FALSE;
+
     out = trealloc(out, char, outN+len+1);
     memcpy(out + outN, t, len);
     outN+=len;
@@ -98,13 +156,37 @@ static char *gen_buf_str_disp()
   char *out = tmalloc(char, 1);
   int outN=0;
 
+  out[0]=0;
   gbuf[ggg.gbufN].ch = " ";
 
+  gboolean last_is_en_word = FALSE;
+
   int N = last_cursor_off ? ggg.gbufN-1:ggg.gbufN;
   for(i=0;i<=N;i++) {
-    char spec[MAX_CIN_PHR * 2];
-    htmlspecialchars(gbuf[i].ch, spec);
-    char www[MAX_CIN_PHR * 2];
+    char addspc[MAX_CIN_PHR * 2 + 2];
+    char spec[MAX_CIN_PHR * 2 + 2];
+    int len = en_word_len(gbuf[i].ch);
+//    dbg("i %d N:%d bufN:%d\n",i,N,ggg.gbufN);
+    if (gbuf[i].flag & FLAG_CHPHO_GTAB_BUF_EN_NO_SPC)
+      len = 0;
+//    dbg("%d %d is_en:%d\n",i, len, last_is_en_word);
+
+    if (len) {
+      if (last_is_en_word) {
+        strcpy(addspc, " ");
+        strcat(addspc, gbuf[i].ch);
+      } else
+        strcpy(addspc, gbuf[i].ch);
+      last_is_en_word = TRUE;
+    } else {
+      last_is_en_word = FALSE;
+      strcpy(addspc, gbuf[i].ch);
+    }
+
+    htmlspecialchars(addspc, spec);
+//    dbg("addspc '%s'  spec:%s out:%s\n", addspc, spec, out);
+
+    char www[MAX_CIN_PHR * 2 + 2];
     char *t = spec;
 
     if (i==ggg.gbuf_cursor) {
@@ -112,13 +194,13 @@ static char *gen_buf_str_disp()
       t = www;
     }
 
-    int len = strlen(t);
+    len = strlen(t);
     out = trealloc(out, char, outN+len+1);
     memcpy(out + outN, t, len);
     outN+=len;
+    out[outN] = 0;
   }
 
-  out[outN] = 0;
   return out;
 }
 
@@ -153,11 +235,14 @@ static void clear_gtab_buf_all()
   disp_gbuf();
 }
 
+
+void minimize_win_gtab();
 void disp_gbuf()
 {
+#if WIN32
   if (test_mode)
     return;
-
+#endif
   char *bf=gen_buf_str_disp();
   disp_label_edit(bf);
 
@@ -165,6 +250,8 @@ void disp_gbuf()
     lookup_gtabn(gbuf[ggg.gbufN-1].ch, NULL);
 
   free(bf);
+
+  minimize_win_gtab();
 }
 
 void clear_gbuf_sel()
@@ -176,12 +263,13 @@ void clear_gbuf_sel()
 
 int gbuf_cursor_left()
 {
+  hide_gtab_pre_sel();
   if (!ggg.gbuf_cursor)
     return ggg.gbufN;
-
+#if WIN32
   if (test_mode)
     return 1;
-
+#endif
   if (ggg.gtab_buf_select)
     clear_gbuf_sel();
   ClrIn();
@@ -193,12 +281,13 @@ int gbuf_cursor_left()
 
 int gbuf_cursor_right()
 {
+  hide_gtab_pre_sel();
   if (ggg.gbuf_cursor==ggg.gbufN)
     return ggg.gbufN;
-
+#if WIN32
   if (test_mode)
     return 1;
-
+#endif
   if (ggg.gtab_buf_select)
     clear_gbuf_sel();
   ggg.gbuf_cursor++;
@@ -208,12 +297,13 @@ int gbuf_cursor_right()
 
 int gbuf_cursor_home()
 {
+  hide_gtab_pre_sel();
   if (!ggg.gbufN)
     return 0;
-
+#if WIN32
   if (test_mode)
     return 1;
-
+#endif
   if (ggg.gtab_buf_select)
     clear_gbuf_sel();
 
@@ -225,12 +315,13 @@ int gbuf_cursor_home()
 
 int gbuf_cursor_end()
 {
+  hide_gtab_pre_sel();
   if (!ggg.gbufN)
     return 0;
-
+#if WIN32
   if (test_mode)
     return 1;
-
+#endif
   if (ggg.gtab_buf_select)
     clear_gbuf_sel();
 
@@ -246,28 +337,30 @@ gboolean output_gbuf()
 {
   if (!ggg.gbufN)
     return FALSE;
-
+#if WIN32
   if (test_mode)
     return TRUE;
-
-  char *bf=gen_buf_str();
+#endif
+  char *bf=gen_buf_str(0, TRUE);
 #if 0
   printf("out %s\n", bf);
 #endif
 
+#if 0
   // single character
   char *p;
   for(p=bf; *p; p+=utf8_sz(p))
     inc_gtab_use_count(p);
+#endif
 
   send_text(bf);
   free(bf);
 
-
   int i;
   for(i=0; i < ggg.gbufN;) {
     char t[MAX_CIN_PHR+1];
     t[0]=0;
+    inc_gtab_use_count(gbuf[i].ch);
 
     int j;
     for(j=i; j < i+gbuf[i].plen; j++)
@@ -390,8 +483,10 @@ void gtab_parse()
         k=0;
       }
 
-      gbuf[ofsi].ch = gbuf[ofsi].sel[k];
-      gbuf[ofsi].c_sel = k;
+      if (!(gbuf[ofsi].flag & FLAG_CHPHO_FIXED)) {
+        gbuf[ofsi].ch = gbuf[ofsi].sel[k];
+        gbuf[ofsi].c_sel = k;
+      }
       gbuf[ofsi].flag |= FLAG_CHPHO_PHRASE_BODY;
 
       ofsi++;
@@ -431,7 +526,7 @@ int qcmp_gitem(const void *aa, const void *bb)
 void hide_row2_if_necessary();
 
 unich_t auto_end_punch[]=_L(", . ? : ; ! [ ] 「 」 ， 。 ？ ； ： 、 ～ ！ （ ）");
-GEDIT *insert_gbuf_cursor(char **sel, int selN, u_int64_t key)
+GEDIT *insert_gbuf_cursor(char **sel, int selN, u_int64_t key, gboolean b_gtab_en_no_spc)
 {
   hide_row2_if_necessary();
 
@@ -439,7 +534,7 @@ GEDIT *insert_gbuf_cursor(char **sel, int selN, u_int64_t key)
     return NULL;
 //  dbg("insert_gbuf_cursor %x\n", key);
 
-  gbuf=trealloc(gbuf, GEDIT, ggg.gbufN+1);
+  gbuf=trealloc(gbuf, GEDIT, ggg.gbufN+2);
 
   GEDIT *pbuf = &gbuf[ggg.gbuf_cursor];
 
@@ -450,6 +545,7 @@ GEDIT *insert_gbuf_cursor(char **sel, int selN, u_int64_t key)
   ggg.gbufN++;
 
   bzero(pbuf, sizeof(GEDIT));
+  bzero(gbuf+ggg.gbufN, sizeof(GEDIT));
 
   GITEM *items = tmalloc(GITEM, selN);
 
@@ -470,6 +566,7 @@ GEDIT *insert_gbuf_cursor(char **sel, int selN, u_int64_t key)
   pbuf->c_sel = 0;
   pbuf->keys[0] = key;
   pbuf->keysN=1;
+  pbuf->flag = b_gtab_en_no_spc ? FLAG_CHPHO_GTAB_BUF_EN_NO_SPC:0;
 
   if (ggg.gbufN==ggg.gbuf_cursor && selN==1 && strstr(_(auto_end_punch), sel[0])) {
     char_play(pbuf->ch);
@@ -491,14 +588,16 @@ void set_gbuf_c_sel(int v)
 
   pbuf->c_sel = v + ggg.pg_idx;
   pbuf->ch = pbuf->sel[pbuf->c_sel];
+//  dbg("zzzsel v:%d %d %s\n",v, pbuf->c_sel,pbuf->ch);
   pbuf->flag |= FLAG_CHPHO_FIXED;
   ggg.gtab_buf_select = 0;
   disp_gtab_sel("");
   gtab_parse();
   disp_gbuf();
+//  dbg("zzzsel v:%d\n", pbuf->c_sel);
 }
 
-GEDIT *insert_gbuf_cursor1(char *s, u_int64_t key)
+GEDIT *insert_gbuf_cursor1(char *s, u_int64_t key, gboolean b_gtab_en_no_spc)
 {
    if (!gtab_phrase_on())
      return NULL;
@@ -506,11 +605,27 @@ GEDIT *insert_gbuf_cursor1(char *s, u_int64_t key)
 //   dbg("insert_gbuf_cursor1 %s %x\n", s, key);
    char **sel = tmalloc(char *, 1);
    sel[0] = strdup(s);
-   GEDIT *e = insert_gbuf_cursor(sel, 1, key);
+   GEDIT *e = insert_gbuf_cursor(sel, 1, key, b_gtab_en_no_spc);
    clear_after_put();
    return e;
 }
 
+void insert_gbuf_cursor_phrase(char *s, void *key, int N)
+{
+  u_int *key32 = (u_int *)key;
+  u_int64_t *key64 = (u_int64_t *)key;
+
+  int i;
+  for(i=0; i < N; i++) {
+    char ch[CH_SZ+1];
+    int n = utf8cpy(ch, s);
+    u_int64_t v = ph_key_sz==4?key32[i]:key64[i];
+    GEDIT *e = insert_gbuf_cursor1(ch, v, TRUE);
+    e->flag |= FLAG_CHPHO_FIXED;
+    s+=n;
+  }
+}
+
 static int key_N(u_int64_t k)
 {
   int n=0;
@@ -535,7 +650,7 @@ static int qcmp_key_N(const void *aa, const void *bb)
 
 void insert_gbuf_nokey(char *s)
 {
-#if WIN32 || 1
+#if WIN32
    if (test_mode)
      return;
 #endif
@@ -571,7 +686,7 @@ void insert_gbuf_nokey(char *s)
 
    qsort(keys, keysN, sizeof(u_int64_t), qcmp_key_N);
 
-   GEDIT *e = insert_gbuf_cursor1(s, keys[0]);
+   GEDIT *e = insert_gbuf_cursor1(s, keys[0], TRUE);
    if (keysN > 8)
      keysN = 8;
 
@@ -581,30 +696,33 @@ void insert_gbuf_nokey(char *s)
 
 void insert_gbuf_cursor1_cond(char *s, u_int64_t key, gboolean valid_key)
 {
+#if WIN32
   if (test_mode)
     return;
+#endif
 
   if (valid_key)
-    insert_gbuf_cursor1(s, key);
+    insert_gbuf_cursor1(s, key, FALSE);
   else
     insert_gbuf_nokey(s);
 }
 
 void insert_gbuf_cursor_char(char ch)
 {
+#if WIN32
   if (test_mode)
     return;
-
+#endif
   char t[2];
   t[0]=ch;
   t[1]=0;
-  insert_gbuf_cursor1(t, 0);
+  insert_gbuf_cursor1(t, 0, TRUE);
 }
 
 gboolean gtab_has_input();
 void hide_win_gtab();
 
-int gtab_buf_delete()
+int gtab_buf_delete_ex(gboolean auto_hide)
 {
   if (ggg.gbuf_cursor==ggg.gbufN)
     return 0;
@@ -620,33 +738,54 @@ int gtab_buf_delete()
   ggg.gbufN--;
   disp_gbuf();
 
-  if (gcin_pop_up_win && !gtab_has_input())
+  if (gcin_pop_up_win && !gtab_has_input() && auto_hide)
     hide_win_gtab();
 
   return 1;
 }
 
+int gtab_buf_delete()
+{
+  return gtab_buf_delete_ex(TRUE);
+}
+
+
 gboolean gtab_has_input();
 void hide_win_gtab();
 
-int gtab_buf_backspace()
+int gtab_buf_backspace_ex(gboolean auto_hide)
 {
   if (!ggg.gbuf_cursor) {
     return ggg.gbufN>0;
   }
 
+#if WIN32
   if (test_mode)
     return 1;
+#endif
 
   ggg.gbuf_cursor--;
-  gtab_buf_delete();
+  gtab_buf_delete_ex(auto_hide);
 
-  if (gcin_pop_up_win && !gtab_has_input())
+  if (gcin_pop_up_win && !gtab_has_input() && auto_hide)
     hide_win_gtab();
 
   return 1;
 }
 
+int gtab_buf_backspace()
+{
+  return gtab_buf_backspace_ex(TRUE);
+}
+
+
+void gtab_buf_backspaceN(int n)
+{
+  int i;
+  for(i=0; i < n; i++)
+    gtab_buf_backspace_ex(FALSE);
+}
+
 extern int more_pg;
 
 void gtab_disp_sel()
@@ -730,32 +869,46 @@ int gtab_get_preedit(char *str, GCIN_PREEDIT_ATTR attr[], int *pcursor, int *sub
   gboolean ap_only = gcin_edit_display_ap_only();
 
   if (gtab_phrase_on()) {
-  attr[0].flag=GCIN_PREEDIT_ATTR_FLAG_UNDERLINE;
-  attr[0].ofs0=0;
+    attr[0].flag=GCIN_PREEDIT_ATTR_FLAG_UNDERLINE;
+    attr[0].ofs0=0;
 
-  if (ggg.gbufN)
-    attrN=1;
+    if (ggg.gbufN)
+      attrN=1;
 
-  for(i=0; i < ggg.gbufN; i++) {
-    char *s = gbuf[i].ch;
-    int len = strlen(s);
-    int N = utf8_str_N(s);
-    ch_N+=N;
-    if (i < ggg.gbuf_cursor)
-      *pcursor+=N;
-    if (i==ggg.gbuf_cursor) {
-      attr[1].ofs0=*pcursor;
-      attr[1].ofs1=*pcursor+N;
-      attr[1].flag=GCIN_PREEDIT_ATTR_FLAG_REVERSE;
-      attrN++;
+    gboolean last_is_en_word = FALSE;
+    for(i=0; i < ggg.gbufN; i++) {
+      char *s = gbuf[i].ch;
+      char tt[MAX_CIN_PHR+2];
+
+      if (en_word_len(s) && !(gbuf[i].flag & FLAG_CHPHO_GTAB_BUF_EN_NO_SPC)) {
+        if (last_is_en_word) {
+          strcpy(tt, " ");
+          strcat(tt, s);
+          s = tt;
+        }
+        last_is_en_word = TRUE;
+      } else {
+        last_is_en_word = FALSE;
+      }
+
+      int len = strlen(s);
+      int N = utf8_str_N(s);
+      ch_N+=N;
+      if (i < ggg.gbuf_cursor)
+        *pcursor+=N;
+      if (i==ggg.gbuf_cursor) {
+        attr[1].ofs0=*pcursor;
+        attr[1].ofs1=*pcursor+N;
+        attr[1].flag=GCIN_PREEDIT_ATTR_FLAG_REVERSE;
+        attrN++;
+      }
+
+      if (gcin_on_the_spot_key && i==ggg.gbuf_cursor)
+        strN += get_DispInArea_str(str+strN);
+
+      memcpy(str+strN, s, len);
+      strN+=len;
     }
-
-    if (ap_only && gcin_on_the_spot_key && i==ggg.gbuf_cursor)
-      strN += get_DispInArea_str(str+strN);
-
-    memcpy(str+strN, s, len);
-    strN+=len;
-  }
   }
 
 
@@ -812,7 +965,7 @@ gboolean save_gtab_buf_shift_enter()
 	int keys[512];
 	int N = ggg.gbufN - ggg.gbuf_cursor;
 	extract_gtab_key(ggg.gbuf_cursor, N, keys);
-	char *str = gen_buf_str();
+	char *str = gen_buf_str(ggg.gbuf_cursor, FALSE);
 	save_phrase_to_db(keys, str, N, 1);
 
 	free(str);
@@ -875,3 +1028,104 @@ void init_tsin_table()
   load_tsin_db0(fname, TRUE);
 #endif
 }
+
+extern u_char scanphr_e(int chpho_idx, int plen, gboolean pho_incr, int *rselN);
+void init_pre_sel();
+
+static int gtab_pre_select_phrase_len;
+
+void disp_gtab_pre_sel(char *s);
+
+void gtab_scan_pre_select(gboolean b_incr)
+{
+  if (!gtab_phrase_pre_select)
+    return;
+//  dbg("gtab_scan_pre_select\n");
+
+  tss.pre_selN = 0;
+
+  hide_gtab_pre_sel();
+
+  if (!gtab_cursor_end() || !ggg.gbufN)
+    return;
+
+  init_tsin_table();
+  init_pre_sel();
+
+  int Maxlen = ggg.gbufN;
+  if (Maxlen > MAX_PHRASE_LEN)
+    Maxlen = MAX_PHRASE_LEN;
+
+  int len, selN, pre_selN=0, max_len=-1, max_selN;
+  for(len=1; len <= Maxlen; len++) {
+    int mlen = scanphr_e(ggg.gbufN - len, len, b_incr, &selN);
+    if (mlen) {
+      max_len = len;
+      max_selN = selN;
+    }
+  }
+
+//  dbg("max_len:%d  max_selN:%d\n", max_len, max_selN);
+
+  if (max_len < 0 || max_selN >= strlen(cur_inmd->selkey) * 2)
+    return;
+
+  gtab_pre_select_phrase_len = max_len;
+
+  scanphr_e(ggg.gbufN - max_len, max_len, b_incr, &selN);
+
+//  dbg("selN:%d %d\n", selN, tss.pre_selN);
+
+  if (selN==1 && tss.pre_sel[0].len==max_len) {
+    char out[MAX_PHRASE_LEN * CH_SZ + 1];
+    extract_gbuf_str(ggg.gbufN - max_len, max_len, out);
+    if (!strcmp(out, tss.pre_sel[0].str))
+      return;
+  }
+
+//  dbg("selN %d %d\n",selN, tss.pre_selN);
+
+  char tt[4096];
+  tt[0]=0;
+  int i;
+  char *br=gtab_vertical_select?"\n":"";
+
+  for(i=0;i<tss.pre_selN; i++) {
+    char ts[(MAX_PHRASE_LEN+3) * CH_SZ + 1];
+    sprintf(ts, "<span foreground=\"%s\">%c</span>%s%s", gcin_sel_key_color,
+      cur_inmd->selkey[i], tss.pre_sel[i].str, br);
+    strcat(tt, ts);
+    if (!gtab_vertical_select && i < tss.pre_selN-1)
+      strcat(tt, " ");
+  }
+
+//  dbg("tt %s\n", tt);
+  disp_gtab_pre_sel(tt);
+}
+
+
+int shift_key_idx(char *s, KeySym xkey);
+
+
+gboolean gtab_pre_select_shift(KeySym key, int kbstate)
+{
+//  dbg("gtab_pre_select_shift %c\n", key);
+  if (!gtab_phrase_pre_select || !tss.pre_selN)
+    return FALSE;
+
+  int c = shift_key_idx(cur_inmd->selkey, key);
+  if (c < 0)
+    return FALSE;
+  if (c >= tss.pre_selN)
+    return TRUE;
+
+#if 0
+  dbg("c %d %s  ggg.gbuf_cursor:%d,%d\n", c, tss.pre_sel[c].str,
+    ggg.gbuf_cursor, ggg.gbufN);
+#endif
+
+  gtab_buf_backspaceN(gtab_pre_select_phrase_len);
+  insert_gbuf_cursor_phrase(tss.pre_sel[c].str, tss.pre_sel[c].phkey, tss.pre_sel[c].len);
+  hide_gtab_pre_sel();
+  return TRUE;
+}
