@@ -21,6 +21,8 @@ GTAB_ST ggg;
 #endif
 
 extern GtkWidget *gwin_gtab;
+void hide_gtab_pre_sel();
+void gtab_scan_pre_select(gboolean);
 
 static GTAB_space_pressed_E _gtab_space_auto_first;
 extern char *TableDir;
@@ -303,6 +305,7 @@ void ClrSelArea()
 {
   disp_gtab_sel("");
   minimize_win_gtab();
+//  hide_gtab_pre_sel();
 }
 
 
@@ -322,8 +325,10 @@ static void clr_seltab()
 void clear_gtab_in_area(), hide_win_gtab();
 void ClrIn()
 {
+#if WIN32
   if (test_mode)
     return;
+#endif
 
   bzero(ggg.inch,sizeof(ggg.inch));
   clr_seltab();
@@ -340,6 +345,7 @@ void ClrIn()
 
   clear_gtab_input_error_color();
   clear_page_label();
+//  hide_gtab_pre_sel();
 }
 
 
@@ -366,10 +372,11 @@ static void DispInArea()
     return;
 #endif
 
-//  dbg("sel1st:%d\n", ggg.sel1st_i);
+//  hide_gtab_pre_sel();
 
-  if (gcin_on_the_spot_key && gcin_edit_display_ap_only()) {
-    if (gwin_gtab && GTK_WIDGET_VISIBLE(gwin_gtab))
+//  dbg("sel1st:%d\n", ggg.sel1st_i);
+  if (gcin_on_the_spot_key) {
+    if (gwin_gtab && GTK_WIDGET_VISIBLE(gwin_gtab) && poo.same_pho_query_state == SAME_PHO_QUERY_none)
       hide_win_gtab();
     return;
   }
@@ -415,6 +422,12 @@ int get_DispInArea_str(char *out)
     }
   }
 
+#if 0
+  if (outN) {
+    hide_gtab_pre_sel();
+  }
+#endif
+
   out[outN]=0;
 //  dbg("get_DispInArea_str\n", out);
   return outN;
@@ -1280,6 +1293,7 @@ static void proc_wild_disp()
 
 gboolean full_char_proc(KeySym keysym);
 void insert_gbuf_cursor_char(char ch);
+gboolean gtab_pre_select_shift(KeySym key, int kbstate);
 
 gboolean shift_char_proc(KeySym key, int kbstate)
 {
@@ -1296,6 +1310,9 @@ gboolean shift_char_proc(KeySym key, int kbstate)
     }
 #endif
 
+    if (gtab_pre_select_shift(key, kbstate))
+      return TRUE;
+
     if (current_CS->b_half_full_char)
       return full_char_proc(key);
 
@@ -1319,13 +1336,13 @@ int gbuf_cursor_right();
 int gbuf_cursor_home();
 int gbuf_cursor_end();
 int gtab_buf_delete();
-GEDIT *insert_gbuf_cursor(char **sel, int selN, u_int64_t key);
 void set_gbuf_c_sel(int v);
 void set_gtab_user_head();
 KeySym keypad_proc(KeySym xkey);
 void save_gtab_buf_phrase(KeySym key);
 gboolean save_gtab_buf_shift_enter();
-
+gboolean win_sym_page_up(), win_sym_page_down();
+u_int64_t vmaskci;
 
 gboolean feedkey_gtab(KeySym key, int kbstate)
 {
@@ -1404,6 +1421,8 @@ shift_proc:
   if (shift_m && !strchr(cur_inmd->selkey, key) && !ggg.more_pg &&
        key!='*' && (key!='?' || gtab_shift_phrase_key && !ggg.ci)) {
     if (gtab_shift_phrase_key) {
+      if (tss.pre_selN && shift_char_proc(key, kbstate))
+        return TRUE;
       if (feed_phrase(key, kbstate))
         return TRUE;
     } else {
@@ -1422,6 +1441,7 @@ shift_proc:
       ggg.spc_pressed=0;
       ggg.sel1st_i=MAX_SELKEY-1;
       clear_gtab_input_error_color();
+      hide_gtab_pre_sel();
 
       if (ggg.ci==0) {
         if (AUTO_SELECT_BY_PHRASE)
@@ -1466,13 +1486,18 @@ shift_proc:
     case XK_KP_Enter:
     case XK_Return:
       if (AUTO_SELECT_BY_PHRASE) {
-		if (shift_m) {
-		  return save_gtab_buf_shift_enter();
-		} else
+        hide_gtab_pre_sel();
+        if (shift_m) {
+          return save_gtab_buf_shift_enter();
+        } else
           return output_gbuf();
       }
       else
         return 0;
+    case XK_Up:
+      if (gtab_has_input())
+        return TRUE;
+      return FALSE;
     case XK_Down:
 #if UNIX
     case XK_KP_Down:
@@ -1482,6 +1507,7 @@ shift_proc:
       else
         return 0;
     case XK_Escape:
+      hide_gtab_pre_sel();
       if (ggg.gtab_buf_select) {
         ggg.gtab_buf_select = 0;
         reset_gtab_all();
@@ -1528,7 +1554,8 @@ shift_proc:
 
       if (key==XK_KP_Subtract)
         goto keypad_proc;
-      return 0;
+
+      return win_sym_page_up();
     case XK_Next:
 #if UNIX
     case XK_KP_Next:
@@ -1548,9 +1575,11 @@ next_page:
       } else {
         if (key==XK_KP_Add)
           goto keypad_proc;
-        return 0;
+        return win_sym_page_down();
       }
     case ' ':
+      hide_gtab_pre_sel();
+
       if (ggg.invalid_spc && gtab_invalid_key_in)
         ClrIn();
 
@@ -1561,7 +1590,7 @@ next_page:
 
       has_wild = has_wild_card();
 
- //     dbg("ggg.wild_mode:%d ggg.more_pg:%d ggg.ci:%d  has_wild:%d\n", ggg.wild_mode, ggg.more_pg, ggg.ci, has_wild);
+//      dbg("ggg.wild_mode:%d ggg.more_pg:%d ggg.ci:%d  has_wild:%d\n", ggg.wild_mode, ggg.more_pg, ggg.ci, has_wild);
 
       if (ggg.wild_mode) {
         // request from tetralet
@@ -1575,6 +1604,7 @@ next_page:
           ggg.wild_page=0;
 
         wildcard();
+        ggg.spc_pressed = TRUE;
         return 1;
       } else
       if (ggg.more_pg && !(_gtab_space_auto_first & GTAB_space_auto_first_any)) {
@@ -1622,6 +1652,7 @@ direct_select:
 
       ggg.last_full=0;
       ggg.spc_pressed=1;
+//      dbg("spc_pressed=1\n");
 
       if (has_wild) {
         ggg.wild_page=0;
@@ -1643,6 +1674,9 @@ direct_select:
           return 0;
       }
     case '*':
+      if (tss.pre_selN && shift_char_proc(key, kbstate))
+        return TRUE;
+
       inkey=cur_inmd->keymap[key];
       if ((inkey && (inkey!=cur_inmd->WILD_STAR && inkey!=cur_inmd->WILD_QUES)) || ptr_selkey(key)) {
 //        dbg("%d %d\n", inkey, cur_inmd->WILD_STAR);
@@ -1837,6 +1871,8 @@ keypad_proc:
 
       if (inkey>=1 && ggg.ci< cur_inmd->MaxPress) {
         ggg.inch[ggg.ci++]=inkey;
+        hide_gtab_pre_sel();
+
         if (gcin_pop_up_win)
           show_win_gtab();
         ggg.last_full=0;
@@ -1930,12 +1966,13 @@ keypad_proc:
 
   int oE1;
   oE1=cur_inmd->idx1[ggg.inch[0]+1];
-  u_int64_t vmaskci;
   if (cur_inmd->keybits==6)
     vmaskci = cur_inmd->key64 ? vmask64[ggg.ci]:vmask[ggg.ci];
   else
     vmaskci = cur_inmd->key64 ? vmask64_7[ggg.ci]:vmask_7[ggg.ci];
 
+  gtab_scan_pre_select(TRUE);
+
   while ((CONVT2(cur_inmd, ggg.S1) & vmaskci) != ggg.kval &&
           CONVT2(cur_inmd, ggg.S1) < ggg.kval &&  ggg.S1<oE1)
     ggg.S1++;
@@ -2097,7 +2134,8 @@ next_pg:
         sel[selN++] = load_tblidx(j);
         j++;
       }
-      insert_gbuf_cursor(sel, selN, ggg.kval);
+      insert_gbuf_cursor(sel, selN, ggg.kval, FALSE);
+      gtab_scan_pre_select(FALSE);
       clear_after_put();
       return 1;
     } else {
