@@ -21,7 +21,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 
-#include "gtkintl.h"
+//#include "gtkintl.h"
 #include <gtk/gtk.h>
 #include "gtkimcontextgcin.h"
 // #include "gcin.h"  // for debug only
@@ -141,10 +141,10 @@ static void
 get_im (GtkIMContextGCIN *context_xim)
 {
   GdkWindow *client_window = context_xim->client_window;
-  GdkScreen *screen = gdk_drawable_get_screen (client_window);
+  GdkScreen *screen = gdk_window_get_screen (client_window);
   GdkDisplay *display = gdk_screen_get_display (screen);
   if (!context_xim->gcin_ch) {
-    if (!(context_xim->gcin_ch = gcin_im_client_open(GDK_DISPLAY())))
+    if (!(context_xim->gcin_ch = gcin_im_client_open(GDK_DISPLAY_XDISPLAY(gdk_display_get_default()))))
       perror("cannot open gcin_ch");
 #if 1
     context_xim->timeout_handle = 0;
@@ -268,39 +268,6 @@ gtk_im_context_gcin_finalize (GObject *obj)
   }
 }
 
-/* Finds the GtkWidget that owns the window, or if none, the
- * widget owning the nearest parent that has a widget.
- */
-
-static GtkWidget *
-widget_for_window (GdkWindow *window)
-{
-  while (window)
-    {
-      gpointer user_data;
-      gdk_window_get_user_data (window, &user_data);
-      if (user_data)
-        return user_data;
-
-      window = gdk_window_get_parent (window);
-    }
-
-  return NULL;
-}
-
-
-#if 0
-static void update_in_toplevel (GtkIMContextGCIN *context_xim)
-{
-}
-static void
-on_client_widget_hierarchy_changed (GtkWidget       *widget,
-                                    GtkWidget       *old_toplevel,
-                                    GtkIMContextGCIN *context_xim)
-{
-  update_in_toplevel (context_xim);
-}
-#endif
 
 static void
 set_ic_client_window (GtkIMContextGCIN *context_xim,
@@ -317,7 +284,7 @@ set_ic_client_window (GtkIMContextGCIN *context_xim,
   if (context_xim->client_window) {
     get_im (context_xim);
     if (context_xim->gcin_ch) {
-      gcin_im_client_set_window(context_xim->gcin_ch, GDK_DRAWABLE_XID(client_window));
+      gcin_im_client_set_window(context_xim->gcin_ch, GDK_WINDOW_XID(client_window));
     }
   }
 }
@@ -327,6 +294,9 @@ static gboolean update_cursor_position(gpointer data)
 {
   GtkIMContextGCIN *context = (GtkIMContextGCIN *)data;
 
+  if (!context)
+    return;
+
   g_signal_emit_by_name(context, "preedit_changed");
   cancel_timeout(context);
   return FALSE;
@@ -373,6 +343,7 @@ gtk_im_context_gcin_new (void)
   return GTK_IM_CONTEXT (result);
 }
 
+#include <gdk/gdkkeysyms.h>
 
 ///
 static gboolean
@@ -389,16 +360,16 @@ gtk_im_context_gcin_filter_keypress (GtkIMContext *context,
   KeySym keysym = 0;
   Status status;
   gboolean result = FALSE;
-  GdkWindow *root_window = gdk_screen_get_root_window (gdk_drawable_get_screen (event->window));
+  GdkWindow *root_window = gdk_screen_get_root_window (gdk_window_get_screen (event->window));
 
   XKeyPressedEvent xevent;
 
   xevent.type = (event->type == GDK_KEY_PRESS) ? KeyPress : KeyRelease;
   xevent.serial = 0;            /* hope it doesn't matter */
   xevent.send_event = event->send_event;
-  xevent.display = GDK_DRAWABLE_XDISPLAY (event->window);
-  xevent.window = GDK_DRAWABLE_XID (event->window);
-  xevent.root = GDK_DRAWABLE_XID (root_window);
+  xevent.display = GDK_WINDOW_XDISPLAY (event->window);
+  xevent.window = GDK_WINDOW_XID (event->window);
+  xevent.root = GDK_WINDOW_XID (root_window);
   xevent.subwindow = xevent.window;
   xevent.time = event->time;
   xevent.x = xevent.x_root = 0;
@@ -406,36 +377,35 @@ gtk_im_context_gcin_filter_keypress (GtkIMContext *context,
   xevent.state = event->state;
   xevent.keycode = event->hardware_keycode;
   xevent.same_screen = True;
-
-  char *rstr = NULL;
   num_bytes = XLookupString (&xevent, buffer, buffer_size, &keysym, NULL);
 
-  // If it is latin key, XLookupString only works for UTF-8 env
+  char *rstr = NULL;
 
 #if (!FREEBSD || MAC_OS)
-  char *lc_ctype = getenv("LC_CTYPE");
-  if (!lc_ctype || !strstr(lc_ctype, "UTF-8")) {
-    int uni = gdk_keyval_to_unicode(event->keyval);
-    if (uni) {
-      gsize rn;
-      GError *err = NULL;
-      char *utf8 = g_convert((char *)&uni, 4, "UTF-8", "UTF-32", &rn, &num_bytes, &err);
+//  if (event->type == GDK_KEY_PRESS)
+//    printf("kval %x %x\n",event->keyval, keysym);
 
-      if (utf8) {
-        strcpy(buffer, utf8);
-        g_free(utf8);
-      }
+  int uni = gdk_keyval_to_unicode(event->keyval);
+  if (uni) {
+    gsize rn;
+    GError *err = NULL;
+    char *utf8 = g_convert((char *)&uni, 4, "UTF-8", "UTF-32", &rn, &num_bytes, &err);
+
+    if (utf8) {
+//      printf("conv %s\n", utf8);
+      strcpy(buffer, utf8);
+      g_free(utf8);
     }
   }
 #endif
 
   gboolean preedit_changed = FALSE;
   gboolean context_has_str = context_xim->pe_str && context_xim->pe_str[0];
-  char *tstr;
+  char *tstr = NULL;
   GCIN_PREEDIT_ATTR att[GCIN_PREEDIT_ATTR_MAX_N];
   int cursor_pos;
 
-  if (xevent.type == KeyPress) {
+  if (event->type == GDK_KEY_PRESS) {
     result = gcin_im_client_forward_key_press(context_xim->gcin_ch,
       keysym, xevent.state, &rstr);
     preedit_changed = result;
@@ -489,7 +459,12 @@ gtk_im_context_gcin_filter_keypress (GtkIMContext *context,
 #if DBG
     printf("seq:%d rstr:%s result:%x num_bytes:%d %x\n", context_xim->gcin_ch->seq, rstr, result, num_bytes, (unsigned int)buffer[0]);
 #endif
-    if (!rstr && !result && num_bytes && buffer[0]>=0x20 && buffer[0]!=0x7f
+    if (!rstr && !result && num_bytes &&
+#if 1
+   buffer[0]>=0x20 && buffer[0]!=0x7f
+#else
+   (event->keyval < 0xf000 || buffer[0]>=0x20 && buffer[0] < 0x7f)
+#endif
         && !(xevent.state & (Mod1Mask|ControlMask))) {
       rstr = (char *)malloc(num_bytes + 1);
       memcpy(rstr, buffer, num_bytes);
