@@ -37,4 +37,14 @@ void prph2(FILE *fp, phokey_t kk)
 void prph(phokey_t kk)
 {
 	prph2(stdout, kk);
-}
\ No newline at end of file
+}
+
+
+void prphs(phokey_t *ks, int ksN)
+{
+  int i;
+  for(i=0;i<ksN;i++) {
+    prph(ks[i]); dbg(" ");
+  }
+}
+
