@@ -4,8 +4,7 @@
 typedef u_short phokey_t;
 
 typedef struct {
-  char selkey[12];
-  int selkeyN;
+  char selkeyN;
   struct {
     char num, typ;
   } phokbm[128][3];  // for 26 keys pho, it may have up-to 3 pho char for a key
@@ -15,7 +14,7 @@ extern PHOKBM phkbm;
 
 typedef struct {
   char ch[CH_SZ];
-  short count;
+  int count;
 } PHO_ITEM;
 
 typedef struct {
@@ -29,9 +28,7 @@ typedef struct {
   phokey_t key;
 } PIN_JUYIN;
 
-extern int start_idx, stop_idx;
-
-#define MAX_PHRASE_LEN (16)
+#define MAX_PHRASE_LEN (32)
 #define MAX_PHRASE_STR_LEN (MAX_PHRASE_LEN * CH_SZ + 1)
 
 #define Min(a,b) ((a) < (b) ? (a):(b))
@@ -52,11 +49,14 @@ extern char *pho_chars[];
 char *phokey_to_str(phokey_t kk);
 int utf8_pho_keys(char *big5, phokey_t *phkeys);
 void prph(phokey_t kk);
+void prphs(phokey_t *ks, int ksN);
 phokey_t pho2key(char typ_pho[]);
-gboolean save_phrase_to_db(phokey_t *phkeys, char *utf8str, int len, usecount_t usecount);
+gboolean save_phrase_to_db(void *phkeys, char *utf8str, int len, usecount_t usecount);
 int lookup(u_char *s);
 int find_match(char *str, int *eq_N, usecount_t *usecount);
 char *phokey_to_str2(phokey_t kk, int last_number);
+char *pho_idx_str(int idx);
+char *pho_idx_str2(int idx, int *is_phrase);
 
 
 #define MAX_PH_BF (90)
@@ -65,9 +65,14 @@ char *phokey_to_str2(phokey_t kk, int last_number);
 
 
 #define TSIN_HASH_SHIFT 6
+#define TSIN_HASH_SHIFT_32 24
+#define TSIN_HASH_SHIFT_64 56
 
 #define PHO_CHAR_LEN 3
 
 #define L_BRACKET_NO 22
 #define R_BRACKET_NO 23
 #define BACK_QUOTE_NO 24
+
+#define PHO_PHRASE_ESCAPE 0x1b
+#define PHO_PINYIN_TONE1 -1
